library benchmarks.src.compiler.compiler_benchmark;

import "package:angular2/bootstrap.dart" show bootstrap;
import "package:angular2/src/platform/browser/browser_adapter.dart"
    show BrowserDomAdapter;
import "package:angular2/src/platform/dom/dom_adapter.dart" show DOM;
import "package:angular2/src/facade/async.dart" show PromiseWrapper;
import "package:angular2/src/facade/collection.dart"
    show ListWrapper, Map, MapWrapper;
import "package:angular2/src/facade/lang.dart"
    show DateWrapper, Type, print, isPresent;
import "package:angular2/core.dart"
    show
        Compiler,
        Component,
        Directive,
        View,
        ViewContainerRef,
        bind,
        provide,
        Provider,
        ViewMetadata;
import "package:angular2/src/core/change_detection/change_detection.dart"
    show ChangeDetectorGenConfig;
import "package:angular2/src/core/linker/view_resolver.dart" show ViewResolver;
import "package:angular2/src/testing/benchmark_util.dart"
    show getIntParameter, bindAction;

List<Provider> _createBindings() {
  var multiplyTemplatesBy = getIntParameter("elements");
  return [
    provide(ViewResolver,
        useFactory: () => new MultiplyViewResolver(multiplyTemplatesBy,
            [BenchmarkComponentNoBindings, BenchmarkComponentWithBindings]),
        deps: []),
    // Use DynamicChangeDetector as that is the only one that Dart supports as well

    // so that we can compare the numbers between JS and Dart
    provide(ChangeDetectorGenConfig,
        useValue: new ChangeDetectorGenConfig(false, false, false))
  ];
}

main() {
  BrowserDomAdapter.makeCurrent();
  bootstrap(CompilerAppComponent, _createBindings()).then((ref) {
    var app = ref.hostComponent;
    bindAction("#compileNoBindings",
        measureWrapper(() => app.compileNoBindings(), "No Bindings"));
    bindAction("#compileWithBindings",
        measureWrapper(() => app.compileWithBindings(), "With Bindings"));
  });
}

measureWrapper(func, desc) {
  return () {
    var begin = DateWrapper.now();
    print('''[${ desc}] Begin...''');
    var onSuccess = (_) {
      var elapsedMs =
          DateWrapper.toMillis(DateWrapper.now()) - DateWrapper.toMillis(begin);
      print('''[${ desc}] ...done, took ${ elapsedMs} ms''');
    };
    var onError = (e) {
      DOM.logError(e);
    };
    PromiseWrapper.then(func(), onSuccess, onError);
  };
}

class MultiplyViewResolver extends ViewResolver {
  num _multiplyBy;
  var _cache = new Map<Type, ViewMetadata>();
  MultiplyViewResolver(num multiple, List<Type> components) : super() {
    /* super call moved to initializer */;
    this._multiplyBy = multiple;
    components.forEach((c) => this._fillCache(c));
  }
  _fillCache(Type component) {
    var view = super.resolve(component);
    var multipliedTemplates = ListWrapper.createFixedSize(this._multiplyBy);
    for (var i = 0; i < this._multiplyBy; ++i) {
      multipliedTemplates[i] = view.template;
    }
    this._cache[component] = new ViewMetadata(
        template: multipliedTemplates.join(""), directives: view.directives);
  }

  ViewMetadata resolve(Type component) {
    var result = this._cache[component];
    return isPresent(result) ? result : super.resolve(component);
  }
}

@Component(selector: "app")
@View(directives: const [], template: '''''')
class CompilerAppComponent {
  Compiler _compiler;
  CompilerAppComponent(this._compiler) {}
  compileNoBindings() {
    this._compiler.clearCache();
    return this._compiler.compileInHost(BenchmarkComponentNoBindings);
  }

  compileWithBindings() {
    this._compiler.clearCache();
    return this._compiler.compileInHost(BenchmarkComponentWithBindings);
  }
}

@Directive(selector: "[dir0]", inputs: const ["prop: attr0"])
class Dir0 {}

@Directive(selector: "[dir1]", inputs: const ["prop: attr1"])
class Dir1 {
  Dir1(Dir0 dir0) {}
}

@Directive(selector: "[dir2]", inputs: const ["prop: attr2"])
class Dir2 {
  Dir2(Dir1 dir1) {}
}

@Directive(selector: "[dir3]", inputs: const ["prop: attr3"])
class Dir3 {
  Dir3(Dir2 dir2) {}
}

@Directive(selector: "[dir4]", inputs: const ["prop: attr4"])
class Dir4 {
  Dir4(Dir3 dir3) {}
}

@Component(selector: "cmp-nobind")
@View(
    directives: const [Dir0, Dir1, Dir2, Dir3, Dir4],
    template: '''
<div class="class0 class1 class2 class3 class4 " nodir0="" attr0="value0" nodir1="" attr1="value1" nodir2="" attr2="value2" nodir3="" attr3="value3" nodir4="" attr4="value4">
  <div class="class0 class1 class2 class3 class4 " nodir0="" attr0="value0" nodir1="" attr1="value1" nodir2="" attr2="value2" nodir3="" attr3="value3" nodir4="" attr4="value4">
    <div class="class0 class1 class2 class3 class4 " nodir0="" attr0="value0" nodir1="" attr1="value1" nodir2="" attr2="value2" nodir3="" attr3="value3" nodir4="" attr4="value4">
      <div class="class0 class1 class2 class3 class4 " nodir0="" attr0="value0" nodir1="" attr1="value1" nodir2="" attr2="value2" nodir3="" attr3="value3" nodir4="" attr4="value4">
        <div class="class0 class1 class2 class3 class4 " nodir0="" attr0="value0" nodir1="" attr1="value1" nodir2="" attr2="value2" nodir3="" attr3="value3" nodir4="" attr4="value4">
        </div>
      </div>
    </div>
  </div>
</div>''')
class BenchmarkComponentNoBindings {}

@Component(selector: "cmp-withbind")
@View(
    directives: const [Dir0, Dir1, Dir2, Dir3, Dir4],
    template: '''
<div class="class0 class1 class2 class3 class4 " dir0="" [attr0]="value0" dir1="" [attr1]="value1" dir2="" [attr2]="value2" dir3="" [attr3]="value3" dir4="" [attr4]="value4">
  {{inter0}}{{inter1}}{{inter2}}{{inter3}}{{inter4}}
  <div class="class0 class1 class2 class3 class4 " dir0="" [attr0]="value0" dir1="" [attr1]="value1" dir2="" [attr2]="value2" dir3="" [attr3]="value3" dir4="" [attr4]="value4">
    {{inter0}}{{inter1}}{{inter2}}{{inter3}}{{inter4}}
    <div class="class0 class1 class2 class3 class4 " dir0="" [attr0]="value0" dir1="" [attr1]="value1" dir2="" [attr2]="value2" dir3="" [attr3]="value3" dir4="" [attr4]="value4">
      {{inter0}}{{inter1}}{{inter2}}{{inter3}}{{inter4}}
      <div class="class0 class1 class2 class3 class4 " dir0="" [attr0]="value0" dir1="" [attr1]="value1" dir2="" [attr2]="value2" dir3="" [attr3]="value3" dir4="" [attr4]="value4">
        {{inter0}}{{inter1}}{{inter2}}{{inter3}}{{inter4}}
        <div class="class0 class1 class2 class3 class4 " dir0="" [attr0]="value0" dir1="" [attr1]="value1" dir2="" [attr2]="value2" dir3="" [attr3]="value3" dir4="" [attr4]="value4">
          {{inter0}}{{inter1}}{{inter2}}{{inter3}}{{inter4}}
        </div>
      </div>
    </div>
  </div>
</div>''')
class BenchmarkComponentWithBindings {}
